==> haystack-0.6.0-dev.0.1%2B2.tar.gz: lib/src/HTime.dart <==
///
/// Copyright (c) 2015, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.htime;

import 'HVal.dart';
import 'ParseError.dart';
import 'package:quiver/core.dart';
import 'package:haystack/src/str-utils.dart';

/// HTime models a time of day tag value.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HTime extends HVal {
  /** Construct with all fields */
  factory HTime(int hour, int min, [int sec = 0, int ms = 0]) {
    if (hour < 0 || hour > 23) throw new FormatException("Invalid hour", hour);
    if (min < 0 || min > 59) throw new FormatException("Invalid min", min);
    if (sec < 0 || sec > 59) throw new FormatException("Invalid sec", sec);
    if (ms < 0 || ms > 999) throw new FormatException("Invalid ms", ms);
    return new HTime._(hour, min, sec, ms);
  }

  /** Construct from Dart DateTime instance */
  factory HTime.fromDateTime(DateTime dt) =>
      new HTime(dt.hour, dt.minute, dt.second, dt.millisecond);

  /** Parse from string fomat "hh:mm:ss.FF" or raise ParseError */
  factory HTime.fromString(String s) {
    ParseError err = new ParseError(s);
    int hour, min, sec, ms;
    try {
      hour = int.parse(s.substring(0, 2));
    } catch (e) {
      throw new ParseError("Invalid hours: $s");
    }
    if (s.codeUnitAt(2) != COLON_CHAR) throw err;
    try {
      min = int.parse(s.substring(3, 5));
    } catch (e) {
      throw new ParseError("Invalid minutes: $s");
    }
    if (s.codeUnitAt(5) != COLON_CHAR) throw err;
    try {
      sec = int.parse(s.substring(6, 8));
    } catch (e) {
      throw new ParseError("invalid seconds: $s");
    }
    if (s.length == "hh:mm:ss".length) return new HTime(hour, min, sec);
    if (s.codeUnitAt(8) != PERIOD_CHAR) throw err;
    ms = 0;
    int pos = 9;
    int places = 0;

    while (pos < s.length) {
      ms = (ms * 10) + (s.codeUnitAt(pos) - '0'.codeUnitAt(0));
      ++pos;
      ++places;
    }

    switch (places) {
      case 1:
        ms *= 100;
        break;
      case 2:
        ms *= 10;
        break;
      case 3:
        break;
      default:
        throw err;
    }

    return new HTime(hour, min, sec, ms);
  }

  /** Singleton for midnight 00:00 */
  static final HTime MIDNIGHT = new HTime._(0, 0, 0, 0);

  HTime._(this.hour, this.min, this.sec, this.ms);

  /** Hour of day as 0-23 */
  final int hour;

  /** Minute of hour as 0-59 */
  final int min;

  /** Second of minute as 0-59 */
  final int sec;

  /** Fractional seconds in milliseconds 0-999 */
  final int ms;

  /** Hash is based on hour, min, sec, ms */
  int _hashCode;
  int get hashCode {
    if (_hashCode == null) {
      _hashCode = hash4(hour, min, sec, ms);
    }
    return _hashCode;
  }

  /** Equals is based on year, month, day */
  bool operator ==(Object that) {
    if (that is! HTime) return false;
    HTime x = that as HTime;
    return hour == x.hour && min == x.min && sec == x.sec && ms == x.ms;
  }

  /** Return sort order as negative, 0, or positive */
  @override
  int compareTo(Object that) {
    HTime x = that as HTime;
    if (hour < x.hour)
      return -1;
    else if (hour > x.hour) return 1;
    if (min < x.min)
      return -1;
    else if (min > x.min) return 1;
    if (sec < x.sec)
      return -1;
    else if (sec > x.sec) return 1;
    if (ms < x.ms)
      return -1;
    else if (ms > x.ms) return 1;

    return 0;
  }

  /** Encode as "h:hh:mm:ss.FFF" */
  String toJson() {
    StringBuffer s = new StringBuffer();
    s.write("h:");
    encode(s);
    return s.toString();
  }

  /** Encode as "hh:mm:ss.FFF" */
  String toZinc() {
    StringBuffer s = new StringBuffer();
    encode(s);
    return s.toString();
  }

  /** Package private implementation shared with HDateTime */
  void encode(StringBuffer s) {
    if (hour < 10) s.write('0');
    s.write(hour);
    s.write(':');
    if (min < 10) s.write('0');
    s.write(min);
    s.write(':');
    if (sec < 10) s.write('0');
    s.write(sec);
    if (ms != 0) {
      s.write('.');
      if (ms < 10) s.write('0');
      if (ms < 100) s.write('0');
      s.write(ms);
    }
  }
}

==> haystack-1.0.0-dev.1.0%2B2.tar.gz: lib/src/HTime.dart <==
///
/// Copyright (c) 2019, BAS Services & Graphics, LLC.
/// Licensed under the Academic Free License version 3.0
///

library haystack.htime;

import 'HVal.dart';
import 'ParseError.dart';
import 'package:quiver/core.dart';
import 'package:haystack/src/string_utils.dart';

/// HTime models a time of day tag value.
///
/// See [Project Haystack](http://project-haystack.org/doc/TagModel#tagKinds)
class HTime extends HVal {
  /// Construct with all fields.
  factory HTime(int hour, int min, [int sec = 0, int ms = 0]) {
    if (hour < 0 || hour > 23) throw new FormatException("Invalid hour", hour);
    if (min < 0 || min > 59) throw new FormatException("Invalid min", min);
    if (sec < 0 || sec > 59) throw new FormatException("Invalid sec", sec);
    if (ms < 0 || ms > 999) throw new FormatException("Invalid ms", ms);
    return new HTime._(hour, min, sec, ms);
  }

  /// Construct from Dart DateTime instance.
  factory HTime.fromDateTime(DateTime dt) =>
      new HTime(dt.hour, dt.minute, dt.second, dt.millisecond);

  /// Parse from string fomat "hh:mm:ss.FF" or raise ParseError.
  factory HTime.fromString(String s) {
    ParseError err = new ParseError(s);
    int hour, min, sec, ms;
    try {
      hour = int.parse(s.substring(0, 2));
    } catch (e) {
      throw new ParseError("Invalid hours: $s");
    }
    if (s.codeUnitAt(2) != COLON_CHAR) throw err;
    try {
      min = int.parse(s.substring(3, 5));
    } catch (e) {
      throw new ParseError("Invalid minutes: $s");
    }
    if (s.codeUnitAt(5) != COLON_CHAR) throw err;
    try {
      sec = int.parse(s.substring(6, 8));
    } catch (e) {
      throw new ParseError("invalid seconds: $s");
    }
    if (s.length == "hh:mm:ss".length) return new HTime(hour, min, sec);
    if (s.codeUnitAt(8) != PERIOD_CHAR) throw err;
    ms = 0;
    int pos = 9;
    int places = 0;

    // HTime only stores ms precision, so truncate anything after that
    while (pos < s.length && places < 3) {
      ms = (ms * 10) + (s.codeUnitAt(pos) - '0'.codeUnitAt(0));
      ++pos;
      ++places;
    }

    switch (places) {
      case 1:
        ms *= 100;
        break;
      case 2:
        ms *= 10;
        break;
      case 3:
        break;
      default:
        throw ParseError("Too much fractional precision: $s");
    }

    return new HTime(hour, min, sec, ms);
  }

  /// Singleton for midnight 00:00.
  static final HTime MIDNIGHT = new HTime._(0, 0, 0, 0);

  HTime._(this.hour, this.min, this.sec, this.ms)
      : hashCode = hash4(hour, min, sec, ms);

  /// Hour of day as 0-23.
  final int hour;

  /// Minute of hour as 0-59.
  final int min;

  /// Second of minute as 0-59.
  final int sec;

  /// Fractional seconds in milliseconds 0-999.
  final int ms;

  /// Hash is based on hour, min, sec, ms.
  @override
  final int hashCode;

  /// Equals is based on year, month, day.
  @override
  bool operator ==(Object that) {
    if (that is! HTime) return false;
    HTime x = that;
    return hour == x.hour && min == x.min && sec == x.sec && ms == x.ms;
  }

  /// Returns sort order as negative, 0, or positive.
  @override
  int compareTo(Object that) {
    HTime x = that as HTime;
    if (hour < x.hour) {
      return -1;
    } else if (hour > x.hour) return 1;

    if (min < x.min) {
      return -1;
    } else if (min > x.min) return 1;

    if (sec < x.sec) {
      return -1;
    } else if (sec > x.sec) return 1;

    if (ms < x.ms) {
      return -1;
    } else if (ms > x.ms) return 1;

    return 0;
  }

  /// Encodes as "h:hh:mm:ss.FFF".
  @override
  String toJson() {
    StringBuffer s = new StringBuffer();
    s.write("h:");
    encode(s);
    return s.toString();
  }

  /// Encodes as "hh:mm:ss.FFF".
  @override
  String toZinc() {
    StringBuffer s = new StringBuffer();
    encode(s);
    return s.toString();
  }

  /// Package private implementation shared with HDateTime.
  void encode(StringBuffer s) {
    if (hour < 10) {
      s.write('0');
    }
    s.write(hour);
    s.write(':');
    if (min < 10) {
      s.write('0');
    }
    s.write(min);
    s.write(':');
    if (sec < 10) {
      s.write('0');
    }
    s.write(sec);
    if (ms != 0) {
      s.write('.');
      if (ms < 10) {
        s.write('0');
      }
      if (ms < 100) {
        s.write('0');
      }
      s.write(ms);
    }
  }
}
